(*--------------------------------------------------------------------------*)
(*                                                                          *)
(*  PASCAL-XSC - MODUL  IEP_IAWA                                  06.05.93  *)
(*                                                                          *)
(*  (c) Markus Neher    Institute for Applied Mathematics                   *)
(*                      D-76128 Karlsruhe University, Germany               *)
(*                      e-mail: ae16@rz.uni-karlsruhe.de                    *)
(*                                                                          *)
(*  INTERVALL-ANFANGSWERTPROBLEMLSER FR DIE STURM-LIOUVILLE-GLEICHUNG     *)
(*                                                                          *)
(*    AUSGANGSPROBLEM :      - U" + P(X)  U(X) =   U(X)                  *)
(*                                                                          *)
(*    TRANSFORMIERT AUF :      U1' = U2                                     *)
(*                             U2' = ( P(X) -  )  U1                      *)
(*                                                                          *)
(*--------------------------------------------------------------------------*)

MODULE iep_iawa;

USE i_ari , mv_ari , mvi_ari , iep_tayk , service , splines;

(*--------------------------------------------------------------------------*)

FUNCTION MIINV2X2( MRA : RMATRIX ) : IMATRIX[1..2,1..2];
                                       
(*  INVERSION EINER 2x2-MATRIX  *)

VAR IDET : INTERVAL;
    MIB  : IMATRIX[1..2,1..2];

BEGIN MIB[1,1] := MRA[2,2];
   MIB[1,2] := - MRA[1,2];
   MIB[2,1] := - MRA[2,1];
   MIB[2,2] :=   MRA[1,1];
   IDET := MIB[1,1] * MIB[2,2] - MIB[1,2] * MIB[2,1];
   MIB[1,1] := MIB[1,1] / IDET;
   MIB[1,2] := MIB[1,2] / IDET;
   MIB[2,1] := MIB[2,1] / IDET;
   MIB[2,2] := MIB[2,2] / IDET;
   MIINV2X2 := MIB;
END;  
     
(*-------------------------------------------------------------------------*)

FUNCTION QR2X2( MRA : RMATRIX ) : RMATRIX[1..2,1..2];
 
(*  QR - ZERLEGUNG EINER 2X2 - MATRIX  *)
  
VAR A , B , R1 , R2 , R3 : REAL;
    MRQ                  : RMATRIX[1..2,1..2];
     
BEGIN A := MRA[1,1];
   B := MRA[2,1];
   R1 := SQRT( SQR( A ) + SQR( B ) );
   IF A > 0 THEN
      R2 := A + R1
   ELSE
      R2 := A - R1;
   R3 := 2 / ( SQR( R2 ) + SQR( B ) );

   MRQ[1,1] := 1 - R3 * SQR( R2 );
   MRQ[1,2] := - B * R2 * R3;
   MRQ[2,1] := MRQ[1,2];
   MRQ[2,2] := 1 - SQR( B ) * R3;
   
   QR2X2 := MRQ;
                                 
END;
   
(*-------------------------------------------------------------------------*)

FUNCTION PERM2X2( MRA : RMATRIX ; VIX : IVECTOR ) : RMATRIX[1..2,1..2];
  
(* PERMUTATION EINER 2X2 - MATRIX, FALLS NEBENBEDINGUNGEN ERFLLT SIND *)
          
VAR R1 , R2 : REAL;
    VRY     : RVECTOR[1..2];
    MRB     : RMATRIX[1..2,1..2];
  
BEGIN R1 := SQR( VIX[1].SUP -> VIX[1].INF );
   R2 := SQR( VIX[2].SUP -> VIX[2].INF );
   VRY[1] := ( SQR( MRA[1,1] ) + SQR( MRA[2,1] ) ) * R1;
   VRY[2] := ( SQR( MRA[1,2] ) + SQR( MRA[2,2] ) ) * R2;
 
  IF VRY[2] <= VRY[1] THEN 
     (* KEIN SPALTENTAUSCH *)
     MRB := MRA
  ELSE BEGIN
      (*  TAUSCHE ERSTE UND ZWEITE SPALTEN VON MRA  *)
      MRB[1,1] := MRA[1,2];
      MRB[2,1] := MRA[2,2];
      MRB[1,2] := MRA[1,1];
      MRB[2,2] := MRA[2,1];
   END;
    
   PERM2X2 := MRB;
                                 
END;
 
(*-------------------------------------------------------------------------*)

FUNCTION TAYKOEFF( TIP: ITKO; VIU: IVECTOR ) : ITVEC[1..2,0..UB(TIP)];

VAR NR , P : INTEGER;
    TIVECU : ITVEC[1..2,0..UB(TIP)];

BEGIN P := UB( TIP );
   TIVECU[1] := TKO( P , VIU[1] );
   TIVECU[2] := TKO( P , VIU[2] );
                                               
   FOR NR := 1 TO P DO BEGIN
       (*  U1' := U2  *)
       TIVECU[1,NR] := TIVECU[2,NR-1] / NR;   
       (*  U2' := FCP * U1  *)
       TIVECU[2,NR] := TIPROD( TIP , TIVECU[1] , NR - 1 ) / NR;
   END; (* FOR *)

   TAYKOEFF := TIVECU;
   
END; 

(*--------------------------------------------------------------------------*)

PROCEDURE PSI_ABLEITUNG( H : INTERVAL; TIP : ITKO;
                         VAR MI_AJ : IMATRIX; VAR TIVECU : ITVEC);

(*  BERECHNET WIRD DIE MATRIX MI_AJ, SIEHE LOHNER SEITE 43  *)
(*  ALS ABFALLPRODUKT ERHLT MAN                            *)
(*  DIE TAYLORKOEFFIZIENTEN DER LSUNG VIU SELBST MIT DAZU  *)

VAR I , J , NR , P : INTEGER;
    IU1 , IU2      : INTERVAL;
    TIMATU         : ITMAT[1..2,1..2,0..UB(TIP)];

BEGIN P := UB( TIP );

   (*  IU1, IU2 ENTHALTEN DEN FUNKTIONSWERT VON U AN DER STELLE TJ         *)
   (*  DIE MATRIX MI_AJ UND DIE LSUNG U DER DGL ERFLLEN DIE GLEICHE DGL  *)
   (*  MI_AJ HAT ANFANGSWERT ID, U(X) HAT ANFANGSWERT (IU1,IU2) !!!        *)

   IU1 := TIVECU[1,0];
   IU2 := TIVECU[2,0];

   TIMATU := TIMATID( P );

   FOR NR := 1 TO P - 1 DO BEGIN

       (*  U11' := U21  *)
       TIMATU[1,1,NR] := TIMATU[2,1,NR-1] / NR;

       (*  U12' := U22  *)
       TIMATU[1,2,NR] := TIMATU[2,2,NR-1] / NR; 

       (*  U21' := FCP * U11  *)
       TIMATU[2,1,NR] := TIPROD( TIP , TIMATU[1,1] , NR - 1 ) / NR;

       (*  U22' := FCP * U12  *)
       TIMATU[2,2,NR] := TIPROD( TIP , TIMATU[1,2] , NR - 1 ) / NR;

       (*  TAYLORKOEFFIZIENTEN VON VIU AUS TAYLORKOEFFIZIENTEN VON MI_AJ !  *)
       TIVECU[1,NR] := TIMATU[1,1,NR] * IU1 + TIMATU[1,2,NR] * IU2;
       TIVECU[2,NR] := TIMATU[2,1,NR] * IU1 + TIMATU[2,2,NR] * IU2;

   END; (* FOR *)
  
   FOR I := 1 TO 2 DO
       FOR J := 1 TO 2 DO BEGIN
           MI_AJ[I,J] := TIMATU[I,J,P-1];
           FOR NR := P - 2 DOWNTO 0 DO
               MI_AJ[I,J] := MI_AJ[I,J] * H + TIMATU[I,J,NR];
       END; (* FOR *)

END; 

(*--------------------------------------------------------------------------*)

PROCEDURE PSI_ABLEITUNG( H : INTERVAL; TIP : ITKO;
                         VAR MI_AJ : IMATRIX; VAR TIVECU, TIVECUINCL : ITVEC);

(*  BERECHNET WIRD DIE MATRIX MI_AJ, SIEHE LOHNER SEITE 43  *)
(*  ALS ABFALLPRODUKT ERHLT MAN                            *)
(*  DIE TAYLORKOEFFIZIENTEN DER LSUNG VIU SELBST MIT DAZU  *)

VAR I , J , NR , P        : INTEGER;
    IU1 , IU2 , IU3 , IU4 : INTERVAL;
    TIMATU                : ITMAT[1..2,1..2,0..UB(TIP)];

BEGIN P := UB( TIP );

   (*  IU1, IU2 ENTHALTEN EINE NHERUNG DES FUNKTIONSWERTS VON U AN DER    *)
   (*  STELLE TJ, IU3 UND IU4 EINE EINSCHLIESSUNG DAVON                    *)
   (*  DIE MATRIX MI_AJ UND DIE LSUNG U DER DGL ERFLLEN DIE GLEICHE DGL  *)
   (*  MI_AJ HAT ANFANGSWERT ID, U(X) HAT ANFANGSWERT (IU1,IU2) !!!        *)

   IU1 := TIVECU[1,0];
   IU2 := TIVECU[2,0];

   IU3 := TIVECUINCL[1,0];
   IU4 := TIVECUINCL[2,0];

   TIMATU := TIMATID( P );

   FOR NR := 1 TO P - 1 DO BEGIN

       (*  U11' := U21  *)
       TIMATU[1,1,NR] := TIMATU[2,1,NR-1] / NR;

       (*  U12' := U22  *)
       TIMATU[1,2,NR] := TIMATU[2,2,NR-1] / NR; 

       (*  U21' := FCP * U11  *)
       TIMATU[2,1,NR] := TIPROD( TIP , TIMATU[1,1] , NR - 1 ) / NR;

       (*  U22' := FCP * U12  *)
       TIMATU[2,2,NR] := TIPROD( TIP , TIMATU[1,2] , NR - 1 ) / NR;

       (*  TAYLORKOEFFIZIENTEN VON VIU AUS TAYLORKOEFFIZIENTEN VON MI_AJ !  *)
       TIVECU[1,NR] := TIMATU[1,1,NR] * IU1 + TIMATU[1,2,NR] * IU2;
       TIVECU[2,NR] := TIMATU[2,1,NR] * IU1 + TIMATU[2,2,NR] * IU2;
       TIVECUINCL[1,NR] := TIMATU[1,1,NR] * IU3 + TIMATU[1,2,NR] * IU4;
       TIVECUINCL[2,NR] := TIMATU[2,1,NR] * IU3 + TIMATU[2,2,NR] * IU4;

   END; (* FOR *)
  
   FOR I := 1 TO 2 DO
       FOR J := 1 TO 2 DO BEGIN
           MI_AJ[I,J] := TIMATU[I,J,P-1];
           FOR NR := P - 2 DOWNTO 0 DO
               MI_AJ[I,J] := MI_AJ[I,J] * H + TIMATU[I,J,NR];
       END; (* FOR *)

END; 

(*--------------------------------------------------------------------------*)

FUNCTION GROBEIN( HI , IP : INTERVAL; VIU : IVECTOR ) : IVECTOR[1..2];

VAR VIY , VIZ : IVECTOR[1..2];

BEGIN VIZ := VIU;
   REPEAT VIY := INTVAL( 0.9 , 1.1 ) * VIZ;
      VIZ[1] := VIU[1] + HI * VIY[2];
      VIZ[2] := VIU[2] + HI * IP * VIY[1];
   UNTIL VIZ <= VIY;

   GROBEIN := VIZ;

END; 

(*-------------------------------------------------------------------------*)

FUNCTION RESTGLIED ( H : INTERVAL; TIP: ITKO; VIU: IVECTOR) : IVECTOR[1..2];

VAR P      : INTEGER;
    VIZ    : IVECTOR[1..2];
    TIVECU : ITVEC[1..2,0..UB(TIP)];

BEGIN P := UB( TIP );
   TIVECU := TAYKOEFF( TIP , VIU );
   VIZ[1] := TIVECU[1,P] * ( H ** P );
   VIZ[2] := TIVECU[2,P] * ( H ** P );
   RESTGLIED := VIZ;
END; 

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION PUNKTAUSWERTUNG( H : INTERVAL; TIU : ITKO ) : INTERVAL;

VAR I , P : INTEGER;
    IX    : INTERVAL;

BEGIN P := UB( TIU );

   IX := TIU[P-1];
   FOR I := P - 2 DOWNTO 0 DO
       IX := H * IX + TIU[I];

   PUNKTAUSWERTUNG := IX;

END; 

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION PUNKTAUSWERTUNG( H : INTERVAL; TIVECU : ITVEC ) : 
                                 IVECTOR[ LB( TIVECU ) .. UB( TIVECU ) ];

VAR I   : INTEGER;
    VIU : IVECTOR[ LB( TIVECU ) .. UB( TIVECU ) ];

BEGIN 

   FOR I := LB( TIVECU ) TO UB( TIVECU ) DO
       VIU[I] := PUNKTAUSWERTUNG( H , TIVECU[I] );

   PUNKTAUSWERTUNG := VIU;

END; 

(*--------------------------------------------------------------------------*)

GLOBAL FUNCTION AWA( UPBD , P : INTEGER; H , EIG : INTERVAL;
       TIVECP , TIVECP_RST : ITVEC ) : IVECTOR[1..2];

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG MIT UNENDLICH OFT          *)
(*  DIFFERENZIERBAREN POTENTIALEN                                          *)
(*  BEZEICHNUNGEN WIE BEI LOHNER !                                         *) 
(*  SEITENANGABEN BEZIEHEN SICH AUF DESSEN DISSERTATION                    *) 

(*  UPBD                 :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ] *)
(*  P                    :  ORDNUNG DER TAYLOR-ENTWICKLUNG                 *)
(*  H , HI               :  SCHRITTWEITE                                   *)
(*  EIG                  :  EIGENWERT                                      *)
(*  TIVECP , TIVECP_RST  :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)        *)

(*  VIU                  :  GESAMTEINSCHLIESSUNG                           *)
(*  VIU_GROB             :  GROBEINSCHLIESSUNGSCHLANGE           SEITE 40  *)
(*  VIU_SCHLANGE         :  INTERVALLGRSSE UJSCHLANGE           SEITE 54  *)
(*  VIU_MID              :  PUNKTGRSSE UJSCHLANGE               SEITE 54  *)
(*  VI_ZJPLUS1           :  DISKRETISIERUNGSFEHLER ZJ+1          SEITE 48  *)
(*  VI_ZJPLUS1_SCHLANGE  :  "  ZJ+1_SCHLANGE                     SEITE 54  *)
(*                             (ENTHLT AUCH DIE RUNDUNGSFEHLER)           *) 
(*  VI_RJ_DACH           :  GLOBALER FEHLER RJ_DACH              SEITE 48  *)
(*  MI_AJ                :  MATRIX AJ                            SEITE 48  *) 
(*  MRB                  :  MATRIX MRBJ BZW MRBJ+1               SEITE 48  *)
(*  MIBINV               :  MRB_JPLUS1_INV                       SEITE 48  *)
                         
VAR I , J , ST                                        : INTEGER;
    T                                                 : REAL;
    HI                                                : INTERVAL;
    VIU , VIU_GROB , VIU_MID , VIU_SCHLANGE , VIU_RST : IVECTOR[1..2];
    VI_ZJPLUS1 , VI_ZJPLUS1_SCHLANGE , VI_RJ_DACH     : IVECTOR[1..2];
    MRB                                               : RMATRIX[1..2,1..2];
    MI_AJ , MI_HILFS , MIBINV                         : IMATRIX[1..2,1..2];
    TIP , TIP_RST                                     : ITKO[0..P];
    TIVECU                                            : ITVEC[1..2,0..P];

BEGIN  HI := INTVAL( 0 , H.SUP );

   VIU[1] := 0;
   VIU[2] := 1;

   VIU_MID := MID( VIU );
   VI_RJ_DACH := VNULL( 2 );
   MRB := ID( 2 );

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   FOR ST := 0 TO UPBD - 1 DO BEGIN

       TIP := TIVECP[ST];
       TIP[0] := TIP[0] - EIG;
       TIP_RST := TIVECP_RST[ST];
       TIP_RST[0] := TIP_RST[0] - EIG;

       VIU_GROB := GROBEIN( HI , TIP_RST[0] , VIU );
       VI_ZJPLUS1 := RESTGLIED( H , TIP_RST , VIU_GROB );
        
       TIVECU[1,0] := VIU_MID[1];
       TIVECU[2,0] := VIU_MID[2];
       
       (*  TAYLOR-KOEFFIZIENTEN VON U SOWIE DIE MATRIX AJ BERECHNEN  *)

       PSI_ABLEITUNG( H , TIP , MI_AJ , TIVECU );
       
       (*  FORMEL (2.4.1) VON LOHNER  *)

       VIU_SCHLANGE := PUNKTAUSWERTUNG( H , TIVECU ) + VI_ZJPLUS1;  
       VIU_MID := MID( VIU_SCHLANGE );
       
       (*  FORMEL (2.4.2) VON LOHNER  *)

       VI_ZJPLUS1_SCHLANGE := VIU_SCHLANGE - VIU_MID;

       MI_HILFS := MI_AJ * MRB;

       (*  FORMEL (2.3.6) VON LOHNER  *)

       MRB := QR2X2( PERM2X2( MID( MI_HILFS ) , VI_RJ_DACH ) );

       MIBINV := MIINV2X2( MRB );
       
       (*  FORMEL (2.3.3) VON LOHNER  *)

       VI_RJ_DACH := (MIBINV * MI_HILFS) * VI_RJ_DACH;
       VI_RJ_DACH := VI_RJ_DACH + MIBINV * VI_ZJPLUS1_SCHLANGE;

       VIU_RST := MRB * VI_RJ_DACH;
       
       (*  FORMEL (2.3.1) VON LOHNER  *)

       VIU := VIU_MID + VIU_RST;  

       WRITE( 'ST :  .....................................................' );
       WRITELN( '...............' , ST : 4 );
        
   END; (* FOR , ST *); 

   AWA := VIU;

END;

(*-------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWA_NWT( UPBD , P : INTEGER; H , EIG : INTERVAL;
       TIVECP , TIVECP_RST : ITVEC; VAR TIMATQ : ITMAT;
       VAR VIU : IVECTOR; VAR VI_U2 : INTERVAL; VAR VI_U2Q : IVECTOR );

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG                            *)
(*  NEWTON-VERFAHREN MIT UNENDLICH OFT DIFFERENZIERBARER BASIS             *)
(*  BEZEICHNUNGEN WIE BEI LOHNER !                                         *) 
(*  SEITENANGABEN BEZIEHEN SICH AUF DESSEN DISSERTATION                    *) 

(*  UPBD                 :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ] *)
(*  P                    :  ORDNUNG DER TAYLOR-ENTWICKLUNG                 *)
(*  H , HI               :  SCHRITTWEITE                                   *)
(*  EIG                  :  EIGENWERT                                      *)
(*  TIVECP , TIVECP_RST  :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)        *)

(*  VIU                  :  GESAMTEINSCHLIESSUNG                           *)
(*  VIU_GROB             :  GROBEINSCHLIESSUNGSCHLANGE           SEITE 40  *)
(*  VIU_SCHLANGE         :  INTERVALLGRSSE UJSCHLANGE           SEITE 54  *)
(*  VIU_MID              :  PUNKTGRSSE UJSCHLANGE               SEITE 54  *)
(*  VI_ZJPLUS1           :  DISKRETISIERUNGSFEHLER ZJ+1          SEITE 48  *)
(*  VI_ZJPLUS1_SCHLANGE  :  "  ZJ+1_SCHLANGE                     SEITE 54  *)
(*                             (ENTHLT AUCH DIE RUNDUNGSFEHLER)           *) 
(*  VI_RJ_DACH           :  GLOBALER FEHLER RJ_DACH              SEITE 48  *)
(*  MI_AJ                :  MATRIX AJ                            SEITE 48  *) 
(*  MRB                  :  MATRIX MRBJ BZW MRBJ+1               SEITE 48  *)
(*  MIBINV               :  MRB_JPLUS1_INV                       SEITE 48  *)
                         
VAR I , NR , ST , N , J                           : INTEGER;
    T                                             : REAL;
    HI , IX                                       : INTERVAL;
    VIU_GROB , VIU_MID , VIU_SCHLANGE , VIU_RST   : IVECTOR[1..2];
    VI_ZJPLUS1 , VI_ZJPLUS1_SCHLANGE , VI_RJ_DACH : IVECTOR[1..2];
    MRB                                           : RMATRIX[1..2,1..2];
    MI_AJ , MI_HILFS , MIBINV                    : IMATRIX[1..2,1..2];
    TIP , TIP_RST                               : ITKO[0..P];
    TI_U2                                      : ITKO[0..2*P];
    TI_U2_IGRAL                               : ITKO[0..2*P+1];
    TIVECU , TIVECUINCL                       : ITVEC[1..2,0..P];
    TIVECQ                                    : ITVEC[0..UB(VI_U2Q),0..P];
    TIVEC_U2Q_IGRAL                           : ITVEC[0..UB(VI_U2Q),0..3*P+1];

BEGIN N := UB( VI_U2Q );
   
   HI := INTVAL( 0 , H.SUP );

   VIU[1] := 0;
   VIU[2] := 1;           
   VI_U2 := 0;
   FOR I := 0 TO N DO
       VI_U2Q[I] := 0;

   VIU_MID := VIU;
   VI_RJ_DACH := VNULL( 2 );
   MRB := ID( 2 );

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   FOR ST := 0 TO UPBD - 1 DO BEGIN

       TIP := TIVECP[ST];
       TIP[0] := TIP[0] - EIG;
       TIP_RST := TIVECP_RST[ST];
       TIP_RST[0] := TIP_RST[0] - EIG;
       FOR I := 0 TO N DO
           TIVECQ[I] := TIMATQ[I,ST];

       VIU_GROB := GROBEIN( HI , TIP_RST[0] , VIU );
       VI_ZJPLUS1 := RESTGLIED( H , TIP_RST , VIU_GROB );
        
       TIVECU[1,0] := VIU_MID[1];
       TIVECU[2,0] := VIU_MID[2];
       TIVECUINCL[1,0] := VIU[1];
       TIVECUINCL[2,0] := VIU[2];
       
       (*  TAYLOR-KOEFFIZIENTEN VON U SOWIE DIE MATRIX AJ BERECHNEN  *)

       PSI_ABLEITUNG( H , TIP , MI_AJ , TIVECU , TIVECUINCL );
           
       TIVECUINCL[1,P] := VI_ZJPLUS1[1] / ( H ** P );

       (*  TAYLOR-KOEFFIZIENTEN VON U^2 UND INTEGRAL DAVON BERECHNEN  *)

       TI_U2_IGRAL[0] := VI_U2;
       FOR NR := 0 TO  P DO BEGIN
           IX := 0;
           FOR I := 0 TO NR DO
               IX := IX + TIVECUINCL[1,I] * TIVECUINCL[1,NR-I];
           TI_U2[NR] := IX;
           TI_U2_IGRAL[NR+1] := TI_U2[NR] / ( NR + 1 );
       END;

       FOR NR := 1 TO P DO BEGIN
           IX := 0;
           FOR I := NR TO P DO
               IX := IX + TIVECUINCL[1,I] * TIVECUINCL[1,P+NR-I];
           TI_U2[P+NR] := IX;
           TI_U2_IGRAL[P+NR+1] := TI_U2[P+NR] / ( P + NR + 1 );
       END;

       VI_U2 := TI_U2_IGRAL[2*P+1];
       FOR I := 2 * P DOWNTO 0 DO
           VI_U2 := H * VI_U2 + TI_U2_IGRAL[I];

       (*  TAYLOR-KOEFFIZIENTEN VON U^2 * Q[I]  (BASIS-POTENTIALE)  *)
       (*  UND INTEGRAL DAVON BERECHNEN                             *)

       FOR I := 0 TO N DO BEGIN
           TIVEC_U2Q_IGRAL[I,0] := VI_U2Q[I];

           FOR NR := 0 TO  P - 1 DO BEGIN
               IX := 0;
               FOR J := 0 TO NR DO
                   IX := IX + TI_U2[J] * TIVECQ[I,NR-J];
               TIVEC_U2Q_IGRAL[I,NR+1] := IX / ( NR + 1 );
           END;

           FOR NR := 0 TO P DO BEGIN
               IX := 0;
               FOR J := 0 TO P DO
                   IX := IX + TI_U2[NR+J] * TIVECQ[I,P-J];
               TIVEC_U2Q_IGRAL[I,P+NR+1] := IX / ( P + NR + 1 );
           END;

           FOR NR := 1 TO P DO BEGIN
               IX := 0;
               FOR J := NR TO P DO
                   IX := IX + TI_U2[2*P+NR-J] * TIVECQ[I,J];
               TIVEC_U2Q_IGRAL[I,2*P+NR+1] := IX / ( 2 * P + NR + 1 );
           END;

       END;    

       FOR I := 0 TO N DO BEGIN
           VI_U2Q[I] := TIVEC_U2Q_IGRAL[I,3*P+1];
           FOR NR := 3 * P DOWNTO 0 DO
               VI_U2Q[I] := H * VI_U2Q[I] + TIVEC_U2Q_IGRAL[I,NR];
       END;
       
       (*  FORMEL (2.4.1) VON LOHNER  *)

       VIU_SCHLANGE := PUNKTAUSWERTUNG( H , TIVECU ) + VI_ZJPLUS1;
       VIU_MID := MID( VIU_SCHLANGE );
       
       (*  FORMEL (2.4.2) VON LOHNER  *)

       VI_ZJPLUS1_SCHLANGE := VIU_SCHLANGE - VIU_MID;

       MI_HILFS := MI_AJ * MRB;

       (*  FORMEL (2.3.6) VON LOHNER  *)

       MRB := QR2X2( PERM2X2( MID( MI_HILFS ) , VI_RJ_DACH ) );

       MIBINV := MIINV2X2( MRB );
       
       (*  FORMEL (2.3.3) VON LOHNER  *)

       VI_RJ_DACH := (MIBINV * MI_HILFS) * VI_RJ_DACH;
       VI_RJ_DACH := VI_RJ_DACH + MIBINV * VI_ZJPLUS1_SCHLANGE;

       VIU_RST := MRB * VI_RJ_DACH;

       (*  FORMEL (2.3.1) VON LOHNER  *)

       VIU := VIU_MID + VIU_RST;

       WRITE( 'ST :  .....................................................' );
       WRITELN( '...............' , ST : 4 );
    
   END; (* FOR , ST *); 

END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION AWA_S( M , DIVI , P : INTEGER; H , EIG : INTERVAL;
       TIVECP , TIVECP_RST : ITVEC ) : IVECTOR[1..2];

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG MIT SPLINE-POTENTIALEN     *)
(*  BEZEICHNUNGEN WIE BEI LOHNER !                                         *)
(*  SEITENANGABEN BEZIEHEN SICH AUF DESSEN DISSERTATION                    *) 

(*  M                    :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ] *)
(*  P                    :  ORDNUNG DER TAYLOR-ENTWICKLUNG                 *)
(*  H , HI               :  SCHRITTWEITE                                   *)
(*  EIG                  :  EIGENWERT                                      *)
(*  TIVECP , TIVECP_RST  :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)        *)

(*  VIU                  :  GESAMTEINSCHLIESSUNG                           *)
(*  VIU_GROB             :  GROBEINSCHLIESSUNGSCHLANGE           SEITE 40  *)
(*  VIU_SCHLANGE         :  INTERVALLGRSSE UJSCHLANGE           SEITE 54  *)
(*  VIU_MID              :  PUNKTGRSSE UJSCHLANGE               SEITE 54  *)

(*  VI_ZJPLUS1           :  DISKRETISIERUNGSFEHLER ZJ+1          SEITE 48  *)
(*  VI_ZJPLUS1_SCHLANGE  :  "  ZJ+1_SCHLANGE                     SEITE 54  *)
(*                             (ENTHLT AUCH DIE RUNDUNGSFEHLER)           *) 
(*  VI_RJ_DACH           :  GLOBALER FEHLER RJ_DACH              SEITE 48  *)
(*  MI_AJ                :  MATRIX AJ                            SEITE 48  *) 
(*  MRB                  :  MATRIX MRBJ BZW MRBJ+1               SEITE 48  *)
(*  MIBINV               :  MRB_JPLUS1_INV                       SEITE 48  *)
                         
VAR I , J , ST , K , L                                : INTEGER;
    T                                                 : REAL;
    HI                                                : INTERVAL;
    VIU , VIU_GROB , VIU_MID , VIU_SCHLANGE , VIU_RST : IVECTOR[1..2];
    VI_Z0_S0                                          : IVECTOR[1..2];
    VI_ZJPLUS1 , VI_ZJPLUS1_SCHLANGE , VI_RJ_DACH     : IVECTOR[1..2];
    MRB , MRC                                         : RMATRIX[1..2,1..2];
    MI_AJ , MI_HILFS , MIBINV                         : IMATRIX[1..2,1..2];
    TIP , TIP_RST                                     : ITKO[0..P];
    TIVECU                                            : ITVEC[1..2,0..P];

BEGIN  HI := INTVAL( 0 , H.SUP );

   VIU[1] := 0;
   VIU[2] := 1;

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   (*  SCHLEIFE BER DIE TEILINTERVALLE  *)

   FOR K := 0 TO M - 1 DO BEGIN

       VIU_MID := MID( VIU );
       VI_Z0_S0 := VIU - VIU_MID;
       MRB := ID( 2 );
       MRC := ID( 2 );

       (*  SCHLEIFE BER DIE STTZSTELLEN IN DEN TEILINTERVALLEN  *)

       FOR L := 0 TO DIVI - 1 DO BEGIN

           ST := K * DIVI + L;

           TIP := TIVECP[ST];
           TIP[0] := TIP[0] - EIG;
           TIP_RST := TIVECP_RST[ST];
           TIP_RST[0] := TIP_RST[0] - EIG;

           VIU_GROB := GROBEIN( HI , TIP_RST[0] , VIU );
           VI_ZJPLUS1 := RESTGLIED( H , TIP_RST , VIU_GROB );

           TIVECU[1,0] := VIU_MID[1];
           TIVECU[2,0] := VIU_MID[2];

           (*  TAYLOR-KOEFFIZIENTEN VON U SOWIE DIE MATRIX AJ BERECHNEN  *)

           PSI_ABLEITUNG( H , TIP , MI_AJ , TIVECU );

           (*  FORMEL (2.3.10) VON LOHNER  *)

           MRC := MID( MI_AJ * MRC );
           IF L = 0 THEN
              VI_RJ_DACH := ( MI_AJ - MRC ) * VI_Z0_S0;

           (*  FORMEL (2.4.1) VON LOHNER  *)

           VIU_SCHLANGE := PUNKTAUSWERTUNG( H , TIVECU ) + VI_ZJPLUS1;
           VIU_MID := MID( VIU_SCHLANGE );

           (*  FORMEL (2.4.2) VON LOHNER  *)

           VI_ZJPLUS1_SCHLANGE := VIU_SCHLANGE - VIU_MID;

           MI_HILFS := MI_AJ * MRB;

           (*  FORMEL (2.3.6) VON LOHNER  *)

           MRB := QR2X2( PERM2X2( MID( MI_HILFS ) , VI_RJ_DACH ) );

           MIBINV := MIINV2X2( MRB );

           (*  FORMEL (2.3.3) VON LOHNER  *)

           VI_RJ_DACH := (MIBINV * MI_HILFS) * VI_RJ_DACH;
           VI_RJ_DACH := VI_RJ_DACH + MIBINV * VI_ZJPLUS1_SCHLANGE;

           VIU_RST := MRB * VI_RJ_DACH;

           (*  FORMEL (2.3.11) VON LOHNER  *)

           VIU := VIU_MID + VIU_RST + MRC * VI_Z0_S0;

           WRITE( 'ST :  .....................................................' );
           WRITELN( '...............' , ST : 4 );

       END; (* FOR , L *);

   END; (* FOR , K *);

   AWA_S := VIU;

END;

(*-------------------------------------------------------------------------*)

GLOBAL FUNCTION AWPS( SPP : ISPLINE; EIG : REAL ) : IVECTOR[1..2];

(*                                                                          *)
(*  ANFANGSWERTL"OSER F"UR STURM-LIOUVILLE-GLEICHUNG MIT SPLINE-POTENTIAL   *)
(*                                                                          *)
(*    AUSGANGSPROBLEM :      - u" + q(x) u =   u                          *)
(*    TRANSFORMIERT AUF :      u" = ( q(x) -  ) u                          *)
(*                                                                          *)
(*  HIER : EINFACHSTE L"OSUNG VON ANFANGSWERTPROBLEMEN                      *)
(*         ENTWICKLUNG VON q(x) IN REIHE                                    *)
(*                                                                          *)
(*         q(x) =  c_0 + c_1 (x-x_i) + c_2 (x-x_i)^2 + c_3 (x-x_i)^3        *)
(*                                                                          *)
(*  POTENZREIHENANSATZ F"UR u(x) F"UHRT AUF REKUSRIONSFORMEL                *)
(*  L"OSUNG MIT VARIABLER TAYLORENTWICKLUNG UND RESTGLIEDABSCH"ATZUNG       *)
(*  IN EINEM EINZIGEN SCHRITT                                               *)

CONST P = 200;            (*  MAXIMALE ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

(*  GLOBALE VARIABLEN :                                           *)
(*                                                                *)
(*     M   : M + 1 = ANZAHL DER ST"UTZSTELLEN DES SPLINES P(X)    *)
(*     N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)  *)

VAR I , J , K , L , M , N , NR             : INTEGER;
    MAXA , MAXC , RESTGLIED , ABLRESTGLIED : REAL;
    PI , PILOW , H , H2 , H3 , IH1         : INTERVAL;
    REIHE , ABLREIHE , IREST , IABLREST    : INTERVAL;
    U , DU                                 : INTERVAL;
    U1 , DU1 , U2 , DU2                    : IVECTOR[0..UB(SPP,2)];
    TIA                                    : IVECTOR[0..P];
    TIC                                    : IVECTOR[0..3];
    VIU                                    : IVECTOR[1..2];
    AKT_DATUM , FUNKT_NAME                 : STRING;
    EIN , DATEIN                           : TEXT;

BEGIN

   M := UB( SPP , 2 );
   N := M DIV 2;

   (* SCHRITTWEITE SETZEN *)

   PI:=INTVAL( RVAL('3.141592653589793115997963468544185161590576171875',-1),
               RVAL('3.141592653589793115997963468544185161590576171875', 1));
   PILOW := INTVAL( RVAL( '1.224646799147353177E-16' , -1 ) ,
                    RVAL( '1.224646799147353177E-16' , +1 ) );
   H  := ( PI + PILOW ) / M;
   H2 := SQR( H );
   H3 := H2 * H;

   (*  SCHLEIFE "UBER ALLE ST"UTZSTELLEN DES SPLINES  *)

   FOR I := 0 TO M - 1 DO BEGIN

       (*  TAYLORKOEFFIZIENTEN VON P(X) BEREITSTELLEN       *)
       (*  BEACHTE, DASS DIE ABLEITUNGEN DES SPLINES        *)
       (*  BEREITS MIT POTENZEN VON h MULTIPLIZIERT WURDEN  *)

       TIC[0] := ( SPP[0,I] - EIG ) * H2;
       TIC[1] := SPP[1,I] * H2;
       TIC[2] := SPP[2,I] * H2 / 2;
       TIC[3] := SPP[3,I] * H2 / 6;

       (*  MINDESTANZAHL DER TERME IN DER TAYLORENTWICKLUNG BERECHNEN  *)

       MAXC := SQRT( SUP( ABS( TIC[0] )
                        + ABS( TIC[1] / H )
                        + ABS( TIC[2] / H2 )
                        + ABS( TIC[3] / H3 ) ) );

       (*  ANFANGSWERTE FESTLEGEN                                 *)
       (*  JEWEILS ZWEI "FUNDAMENTAL"-ANFANGSWERTPROBLEME L"OSEN  *)
       (*  UND DANACH GEEIGNETE LINEARKOMBINATIONEN BILDEN        *)

       TIA[0] := 1;
       TIA[1] := 0;

       (*  "VORITERATION"  *)

       TIA[2] := TIC[0] * TIA[0] / 2;
       TIA[3] := ##( TIC[0] * TIA[1] + TIC[1] * TIA[0] ) / 6;
       TIA[4] := ##( TIC[0] * TIA[2] + TIC[1] * TIA[1]
                   + TIC[2] * TIA[0] ) / 12;

       REIHE := ##( FOR J := 0 TO 4 SUM( TIA[J] ) );
       ABLREIHE := ##( FOR J := 1 TO 4 SUM( J * TIA[J] ) );

       (*  WERTE ZUM EINSPRUNG IN SCHLEIFE VORBESETZEN  *)

       RESTGLIED := SUP( ABS( REIHE ) );
       ABLRESTGLIED := SUP( ABS( ABLREIHE ) );

       NR := 4;

       WHILE ( ( NR < MAXC ) OR ( RESTGLIED /> SUP( ABS( REIHE ) )
             +> ABLRESTGLIED /> SUP( ABS(  ABLREIHE ) ) > 1.0E-16 ) ) DO BEGIN

             NR := NR + 1;

             TIA[NR] := ##( FOR J := 0 TO 3
                                  SUM( TIC[J] * TIA[NR-2-J] ) )
                                       / ( NR * ( NR - 1 ) );

             REIHE := REIHE + TIA[NR];
             ABLREIHE := ABLREIHE + NR * TIA[NR];

             (* RESTGLIED ABSCH"ATZEN  *)
             (* ZUERST MAXIMUM DER LETZTEN F"UNF KOEFFIZIENTEN BESTIMMEN *)

             MAXA := SUP( ABS( TIA[NR] ) );
             FOR J := 1 TO 4 DO
                 IF SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) ) > MAXA THEN
                    MAXA := SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) );

             IH1 := H / ( 1 - H );
             RESTGLIED := MAXA *> IH1.SUP;
             IH1 := ( 1 + NR * ( 1 - H ) ) / SQR( 1 - H );
             ABLRESTGLIED := MAXA *> IH1.SUP;

       END;

       WRITELN( 'Restglied [' , NR : 2 , ']      = ' , RESTGLIED );
       WRITELN( 'Ableitungsrestglied = ' , ABLRESTGLIED );
       WRITELN;

       IREST    := INTVAL( - RESTGLIED    , RESTGLIED );
       IABLREST := INTVAL( - ABLRESTGLIED , ABLRESTGLIED );

       REIHE    := ##( FOR J := 0 TO NR SUM( TIA[J] ) + IREST );
       ABLREIHE := ##( FOR J := 1 TO NR SUM( J * TIA[J] ) + IABLREST );
       WRITE(   'Reihe            =  ' );
       IWRITELN( REIHE );
       WRITE(   'Ableitungsreihe  =  ' );
       IWRITELN( ABLREIHE );
       WRITELN;

       (*  WERTE ABSPEICHERN  *)

       U1[I]  := REIHE;
       DU1[I] := ABLREIHE;

       (*  ANFANGSWERTE DER ZWEITEN "FUNDAMENTAL"-L"OSUNG FESTLEGEN  *)

       TIA[0] := 0;
       TIA[1] := 1;

       (*  "VORITERATION"  *)

       TIA[2] := TIC[0] * TIA[0] / 2;
       TIA[3] := ##( TIC[0] * TIA[1] + TIC[1] * TIA[0] ) / 6;
       TIA[4] := ##( TIC[0] * TIA[2] + TIC[1] * TIA[1]
                   + TIC[2] * TIA[0] ) / 12;

       REIHE := ##( FOR J := 0 TO 4 SUM( TIA[J] ) );
       ABLREIHE := ##( FOR J := 1 TO 4 SUM( J * TIA[J] ) );

       (*  WERTE ZUM EINSPRUNG IN SCHLEIFE VORBESETZEN  *)

       RESTGLIED := SUP( ABS( REIHE ) );
       ABLRESTGLIED := SUP( ABS( ABLREIHE ) );

       NR := 4;

       WHILE ( ( NR < MAXC ) OR ( RESTGLIED /> SUP( ABS( REIHE ) )
             +> ABLRESTGLIED /> SUP( ABS(  ABLREIHE ) ) > 1.0E-16 ) ) DO BEGIN

             NR := NR + 1;

             TIA[NR] := ##( FOR J := 0 TO 3
                                  SUM( TIC[J] * TIA[NR-2-J] ) )
                                       / ( NR * ( NR - 1 ) );

             REIHE := REIHE + TIA[NR];
             ABLREIHE := ABLREIHE + NR * TIA[NR];

             (* RESTGLIED ABSCH"ATZEN  *)
             (* ZUERST MAXIMUM DER LETZTEN F"UNF KOEFFIZIENTEN BESTIMMEN *)

             MAXA := SUP( ABS( TIA[NR] ) );
             FOR J := 1 TO 4 DO
                 IF SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) ) > MAXA THEN
                    MAXA := SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) );

             IH1 := H / ( 1 - H );
             RESTGLIED := MAXA *> IH1.SUP;
             IH1 := ( 1 + NR * ( 1 - H ) ) / SQR( 1 - H );
             ABLRESTGLIED := MAXA *> IH1.SUP;

       END;

       WRITELN( 'Restglied [' , NR : 2 , ']      = ' , RESTGLIED );
       WRITELN( 'Ableitungsrestglied = ' , ABLRESTGLIED );
       WRITELN;

       IREST    := INTVAL( - RESTGLIED    , RESTGLIED );
       IABLREST := INTVAL( - ABLRESTGLIED , ABLRESTGLIED );

       REIHE    := ##( FOR J := 0 TO NR SUM( TIA[J] ) + IREST );
       ABLREIHE := ##( FOR J := 1 TO NR SUM( J * TIA[J] ) + IABLREST );
       WRITE(   'Reihe            =  ' );
       IWRITELN( REIHE );
       WRITE(   'Ableitungsreihe  =  ' );
       IWRITELN( ABLREIHE );
       WRITELN;

       (*  WERTE ABSPEICHERN  *)

       U2[I]  := REIHE;
       DU2[I] := ABLREIHE;

   END; (* FOR , I *)

   (*  JETZT BERECHNETE WERTE MULTIPLIZIEREN UND AUSGEBEN  *)

   U  := U2[0];
   DU := DU2[0];

   WRITELN( 'Einschlieungen des Anfangswertproblems :' );
   WRITELN;
   WRITELN( 'u ( 0  / ' , M : 1 , ' ) = 0' );
   WRITELN( 'u''( 0  / ' , M : 1 , ' ) = 1' );
   WRITELN;
   WRITE(   'u ( 1  / ' , M : 1 , ' ) = ' );
   IWRITELN( U );
   WRITE(   'u''( 1  / ' , M : 1 , ' ) = ' );
   IWRITELN( DU );
   WRITELN;

   FOR I := 1 TO M - 1 DO BEGIN

       IH1 := U * U1[I]  + DU * U2[I];
       DU  := U * DU1[I] + DU * DU2[I];
       U   := IH1;

       WRITE(   'u (' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ' );
       IWRITELN( U );
       WRITE(   'u''(' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ' );
       IWRITELN( DU );
       WRITELN;

   END;

   VIU[1] := U;
   VIU[2] := DU;

   AWPS := VIU;

END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWA_SNWT( M , DIVI , P : INTEGER; H , EIG : INTERVAL;
       TIVECP , TIVECP_RST : ITVEC; VAR TIMATQ : ITMAT;
       VAR VIU : IVECTOR; VAR VI_U2 : INTERVAL; VAR VI_U2Q : IVECTOR );

(*  AWP-LSER FR DIE STURM-LIOUVILLE-GLEICHUNG                            *)
(*  NEWTON-VERFAHREN MIT SPLINE-BASIS                                      *)
(*  BEZEICHNUNGEN WIE BEI LOHNER !                                         *) 
(*  SEITENANGABEN BEZIEHEN SICH AUF DESSEN DISSERTATION                    *) 

(*  M                    :  ANZAHL DER STTZSTELLEN IM INTERVALL [ 0 ,  ] *)
(*  P                    :  ORDNUNG DER TAYLOR-ENTWICKLUNG                 *)
(*  H , HI               :  SCHRITTWEITE                                   *)
(*  EIG                  :  EIGENWERT                                      *)
(*  TIVECP , TIVECP_RST  :  TAYLORKOEFFIZIENTEN DES POTENTIALS P(X)        *)

(*  VIU                  :  GESAMTEINSCHLIESSUNG                           *)
(*  VIU_GROB             :  GROBEINSCHLIESSUNGSCHLANGE           SEITE 40  *)
(*  VIU_SCHLANGE         :  INTERVALLGRSSE UJSCHLANGE           SEITE 54  *)
(*  VIU_MID              :  PUNKTGRSSE UJSCHLANGE               SEITE 54  *)
(*  VI_ZJPLUS1           :  DISKRETISIERUNGSFEHLER ZJ+1          SEITE 48  *)
(*  VI_ZJPLUS1_SCHLANGE  :  "  ZJ+1_SCHLANGE                     SEITE 54  *)
(*                             (ENTHLT AUCH DIE RUNDUNGSFEHLER)           *) 
(*  VI_RJ_DACH           :  GLOBALER FEHLER RJ_DACH              SEITE 48  *)
(*  MI_AJ                :  MATRIX AJ                            SEITE 48  *) 
(*  MRB                  :  MATRIX MRBJ BZW MRBJ+1               SEITE 48  *)
(*  MIBINV               :  MRB_JPLUS1_INV                       SEITE 48  *)
                         
VAR I , NR , ST , N , J , K , L                   : INTEGER;
    T                                             : REAL;
    HI , IX                                       : INTERVAL;
    VIU_GROB , VIU_MID , VIU_SCHLANGE , VIU_RST   : IVECTOR[1..2];
    VI_Z0_S0                                      : IVECTOR[1..2];
    VI_ZJPLUS1 , VI_ZJPLUS1_SCHLANGE , VI_RJ_DACH : IVECTOR[1..2];
    MRB , MRC                                     : RMATRIX[1..2,1..2];
    MI_AJ , MI_HILFS , MIBINV                    : IMATRIX[1..2,1..2];
    TIP , TIP_RST                               : ITKO[0..P];
    TI_U2                                      : ITKO[0..2*P];
    TI_U2_IGRAL                               : ITKO[0..2*P+1];
    TIVECU , TIVECUINCL                       : ITVEC[1..2,0..P];
    TIVECQ                                    : ITVEC[0..UB(VI_U2Q),0..P];
    TIVEC_U2Q_IGRAL                           : ITVEC[0..UB(VI_U2Q),0..3*P+1];

BEGIN N := UB( VI_U2Q );
   
   HI := INTVAL( 0 , H.SUP );

   VIU[1] := 0;
   VIU[2] := 1;

   VI_U2 := 0;
   FOR I := 0 TO N DO
       VI_U2Q[I] := 0;

   (*  JETZT STARTET DAS LSUNGSVERFAHREN DER DGL  *)
   
   (*  SCHLEIFE BER DIE TEILINTERVALLE  *)

   FOR K := 0 TO M - 1 DO BEGIN

       VIU_MID := MID( VIU );
       VI_Z0_S0 := VIU - VIU_MID;
       MRB := ID( 2 );
       MRC := ID( 2 );

       (*  SCHLEIFE BER DIE STTZSTELLEN IN DEN TEILINTERVALLEN  *)

       FOR L := 0 TO DIVI - 1 DO BEGIN

           ST := K * DIVI + L;

           TIP := TIVECP[ST];
           TIP[0] := TIP[0] - EIG;
           TIP_RST := TIVECP_RST[ST];
           TIP_RST[0] := TIP_RST[0] - EIG;
           FOR I := 0 TO N DO
               TIVECQ[I] := TIMATQ[I,ST];

           VIU_GROB := GROBEIN( HI , TIP_RST[0] , VIU );
           VI_ZJPLUS1 := RESTGLIED( H , TIP_RST , VIU_GROB );

           TIVECU[1,0] := VIU_MID[1];
           TIVECU[2,0] := VIU_MID[2];
           TIVECUINCL[1,0] := VIU[1];
           TIVECUINCL[2,0] := VIU[2];

           (*  TAYLOR-KOEFFIZIENTEN VON U SOWIE DIE MATRIX AJ BERECHNEN  *)

           PSI_ABLEITUNG( H , TIP , MI_AJ , TIVECU , TIVECUINCL );

           (*  FORMEL (2.3.10) VON LOHNER  *)

           MRC := MID( MI_AJ * MRC );
           IF L = 0 THEN
              VI_RJ_DACH := ( MI_AJ - MRC ) * VI_Z0_S0;

           (*  TAYLOR-KOEFFIZIENTEN VON U^2 UND INTEGRAL DAVON BERECHNEN  *)

           TIVECUINCL[1,P] := VI_ZJPLUS1[1] / ( H ** P );

           TI_U2_IGRAL[0] := VI_U2;
           FOR NR := 0 TO  P DO BEGIN
               IX := 0;
               FOR I := 0 TO NR DO
                   IX := IX + TIVECUINCL[1,I] * TIVECUINCL[1,NR-I];
               TI_U2[NR] := IX;
               TI_U2_IGRAL[NR+1] := TI_U2[NR] / ( NR + 1 );
           END;

           FOR NR := 1 TO P DO BEGIN
               IX := 0;
               FOR I := NR TO P DO
                   IX := IX + TIVECUINCL[1,I] * TIVECUINCL[1,P+NR-I];
               TI_U2[P+NR] := IX;
               TI_U2_IGRAL[P+NR+1] := TI_U2[P+NR] / ( P + NR + 1 );
           END;

           VI_U2 := TI_U2_IGRAL[2*P+1];
           FOR I := 2 * P DOWNTO 0 DO
               VI_U2 := H * VI_U2 + TI_U2_IGRAL[I];

           (*  TAYLOR-KOEFFIZIENTEN VON U^2 * Q[I]  (BASIS-SPLINES)  *)
           (*  UND INTEGRAL DAVON BERECHNEN                          *)

           FOR I := 0 TO N DO BEGIN
               TIVEC_U2Q_IGRAL[I,0] := VI_U2Q[I];

               FOR NR := 0 TO  P - 1 DO BEGIN
                   IX := 0;
                   FOR J := 0 TO NR DO
                       IX := IX + TI_U2[J] * TIVECQ[I,NR-J];
                   TIVEC_U2Q_IGRAL[I,NR+1] := IX / ( NR + 1 );
               END;

               FOR NR := 0 TO P DO BEGIN
                   IX := 0;
                   FOR J := 0 TO P DO
                       IX := IX + TI_U2[NR+J] * TIVECQ[I,P-J];
                   TIVEC_U2Q_IGRAL[I,P+NR+1] := IX / ( P + NR + 1 );
               END;

               FOR NR := 1 TO P DO BEGIN
                   IX := 0;
                   FOR J := NR TO P DO
                       IX := IX + TI_U2[2*P+NR-J] * TIVECQ[I,J];
                   TIVEC_U2Q_IGRAL[I,2*P+NR+1] := IX / ( 2 * P + NR + 1 );
               END;

           END;

           FOR I := 0 TO N DO BEGIN
               VI_U2Q[I] := TIVEC_U2Q_IGRAL[I,3*P+1];
               FOR NR := 3 * P DOWNTO 0 DO
                   VI_U2Q[I] := H * VI_U2Q[I] + TIVEC_U2Q_IGRAL[I,NR];
           END;

           (*  FORMEL (2.4.1) VON LOHNER  *)

           VIU_SCHLANGE := PUNKTAUSWERTUNG( H , TIVECU ) + VI_ZJPLUS1;
           VIU_MID := MID( VIU_SCHLANGE );

           (*  FORMEL (2.4.2) VON LOHNER  *)

           VI_ZJPLUS1_SCHLANGE := VIU_SCHLANGE - VIU_MID;

           MI_HILFS := MI_AJ * MRB;

           (*  FORMEL (2.3.6) VON LOHNER  *)

           MRB := QR2X2( PERM2X2( MID( MI_HILFS ) , VI_RJ_DACH ) );

           MIBINV := MIINV2X2( MRB );

           (*  FORMEL (2.3.3) VON LOHNER  *)

           VI_RJ_DACH := (MIBINV * MI_HILFS) * VI_RJ_DACH;
           VI_RJ_DACH := VI_RJ_DACH + MIBINV * VI_ZJPLUS1_SCHLANGE;

           VIU_RST := MRB * VI_RJ_DACH;

           (*  FORMEL (2.3.11) VON LOHNER  *)

           VIU := VIU_MID + VIU_RST + MRC * VI_Z0_S0;

           WRITE( 'ST :  .....................................................' );
           WRITELN( '...............' , ST : 4 );
    
       END; (* FOR , L *);

   END; (* FOR , K *);

END;

(*--------------------------------------------------------------------------*)

GLOBAL PROCEDURE AWPS_NWT( SPP : ISPLINE; EIG : INTERVAL; SPQ : ISVECTOR;
       VAR VIU : IVECTOR; VAR VI_U2 : INTERVAL; VAR VI_U2Q : IVECTOR );

(*                                                                          *)
(*  ANFANGSWERTL"OSER F"UR STURM-LIOUVILLE-GLEICHUNG MIT SPLINE-POTENTIAL   *)
(*                                                                          *)
(*    AUSGANGSPROBLEM :      - u" + q(x) u =   u                          *)
(*    TRANSFORMIERT AUF :      u" = ( q(x) -  ) u                          *)
(*                                                                          *)
(*  HIER : EINFACHSTE L"OSUNG VON ANFANGSWERTPROBLEMEN                      *)
(*         ENTWICKLUNG VON q(x) IN REIHE                                    *)
(*                                                                          *)
(*         q(x) =  c_0 + c_1 (x-x_i) + c_2 (x-x_i)^2 + c_3 (x-x_i)^3        *)
(*                                                                          *)
(*  POTENZREIHENANSATZ F"UR u(x) F"UHRT AUF REKUSRIONSFORMEL                *)
(*  L"OSUNG MIT VARIABLER TAYLORENTWICKLUNG UND RESTGLIEDABSCH"ATZUNG       *)
(*  IN EINEM EINZIGEN SCHRITT                                               *)

CONST P = 200;            (*  MAXIMALE ORDNUNG DER TAYLORENTWICKLUNG VON U  *)

(*  GLOBALE VARIABLEN :                                           *)
(*                                                                *)
(*     M   : M + 1 = ANZAHL DER ST"UTZSTELLEN DES SPLINES P(X)    *)
(*     N   : N + 1 = ANZAHL DER VORGEGEBENEN EIGENWERTE VON P(X)  *)

VAR EW , M , N , I , J , K , L , NR             : INTEGER;
    MAXA , MAXC , RESTGLIED , ABLRESTGLIED , R1 : REAL;
    PI , PILOW , H , H2 , H3 , IH1              : INTERVAL;
    REIHE , ABLREIHE , IREST , IABLREST         : INTERVAL;
    TIC                                         : IVECTOR[0..3];
    TIA , TIU2 , TIU2INT                        : IVECTOR[0..P];
    TIU2Q                                       : IMATRIX[0..UB(VI_U2Q),0..P];

(* TIA   ENTHLT DIE SUMMANDEN DER POTENZREIHE VON u (MIT POTENZEN VON h^j) *)
(* TIU2  ENTHLT DIE SUMMANDEN VON u^2                                      *)
(* TIU2INT ENTHLT DIE SUMMANDEN VON \int u^2 dx                            *)
(* TIU2Q ENTHLT DIE SUMMANDEN VON \int u^2 q_j dx                          *)

BEGIN

   M := UB( SPP , 2 );
   N := M DIV 2;

   (* SCHRITTWEITE SETZEN *)

   PI:=INTVAL( RVAL('3.141592653589793115997963468544185161590576171875',-1),
               RVAL('3.141592653589793115997963468544185161590576171875', 1));
   PILOW := INTVAL( RVAL( '1.224646799147353177E-16' , -1 ) ,
                    RVAL( '1.224646799147353177E-16' , +1 ) );
   H  := ( PI + PILOW ) / M;
   H2 := SQR( H );
   H3 := H2 * H;

   (*  SCHLEIFE "UBER ALLE ST"UTZSTELLEN DES SPLINES  *)

   (*  ANFANGSWERTE FESTLEGEN  *)

   REIHE := 0;
   ABLREIHE := 1;
   VI_U2 := 0;
   FOR J := 0 TO N DO
       VI_U2Q[J] := 0;

   FOR I := 0 TO M - 1 DO BEGIN

       (*  TAYLORKOEFFIZIENTEN VON P(X) BEREITSTELLEN       *)
       (*  BEACHTE, DASS DIE ABLEITUNGEN DES SPLINES        *)
       (*  BEREITS MIT POTENZEN VON h MULTIPLIZIERT WURDEN  *)

       TIC[0] := ( SPP[0,I] - EIG ) * H2;
       TIC[1] := SPP[1,I] * H2;
       TIC[2] := SPP[2,I] * H2 / 2;
       TIC[3] := SPP[3,I] * H2 / 6;
       FOR K := 0 TO N DO BEGIN
           SPQ[K,2,I] := SPQ[K,2,I] / 2;
           SPQ[K,3,I] := SPQ[K,3,I] / 6;
       END;

       (*  MINDESTANZAHL DER TERME IN DER TAYLORENTWICKLUNG BERECHNEN  *)

       MAXC := SQRT( SUP( ABS( TIC[0] )
                        + ABS( TIC[1] / H )
                        + ABS( TIC[2] / H2 )
                        + ABS( TIC[3] / H3 ) ) );

       (*  ANFANGSWERTE FESTLEGEN  *)

       TIA[0] := REIHE;
       TIA[1] := ABLREIHE;

       (*  "VORITERATION"  *)

       TIA[2] := TIC[0] * TIA[0] / 2;
       TIA[3] := ##( TIC[0] * TIA[1] + TIC[1] * TIA[0] ) / 6;
       TIA[4] := ##( TIC[0] * TIA[2] + TIC[1] * TIA[1]
                   + TIC[2] * TIA[0] ) / 12;

       REIHE := ##( FOR J := 0 TO 4 SUM( TIA[J] ) );
       ABLREIHE := ##( FOR J := 1 TO 4 SUM( J * TIA[J] ) );

       (*  WERTE ZUM EINSPRUNG IN SCHLEIFE VORBESETZEN  *)

       RESTGLIED := SUP( ABS( REIHE ) );
       ABLRESTGLIED := SUP( ABS( ABLREIHE ) );

       NR := 4;

       WHILE ( ( NR < MAXC ) OR ( RESTGLIED /> SUP( ABS( REIHE ) )
             +> ABLRESTGLIED /> SUP( ABS(  ABLREIHE ) ) > 1.0E-16 ) ) DO BEGIN

             NR := NR + 1;

             TIA[NR] := ##( FOR J := 0 TO 3
                                  SUM( TIC[J] * TIA[NR-2-J] ) )
                                       / ( NR * ( NR - 1 ) );

             REIHE := REIHE + TIA[NR];
             ABLREIHE := ABLREIHE + NR * TIA[NR];

             (* RESTGLIED ABSCH"ATZEN  *)
             (* ZUERST MAXIMUM DER LETZTEN F"UNF KOEFFIZIENTEN BESTIMMEN *)

             MAXA := SUP( ABS( TIA[NR] ) );
             FOR J := 1 TO 4 DO
                 IF SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) ) > MAXA THEN
                    MAXA := SUP( ( H ** J ) * ( ABS( TIA[NR-J] ) ) );

             IH1 := H / ( 1 - H );
             RESTGLIED := MAXA *> IH1.SUP;
             IH1 := ( 1 + NR * ( 1 - H ) ) / SQR( 1 - H );
             ABLRESTGLIED := MAXA *> IH1.SUP;

       END;

       IREST    := INTVAL( - RESTGLIED    , RESTGLIED );
       IABLREST := INTVAL( - ABLRESTGLIED , ABLRESTGLIED );

       REIHE    := ##( FOR J := 0 TO NR SUM( TIA[J] ) + IREST );
       ABLREIHE := ##( FOR J := 1 TO NR SUM( J * TIA[J] ) + IABLREST );

       (* JETZT REIHEN FR u^2, \int u^2 dx, \int u^2 q_j dx  BERECHNEN     *)
       (* BEI DIESER BERECHNUNG KANN DAS RESTGLIED ZU TIA[0] ADDIERT WERDEN *)

       TIA[0] := TIA[0] + IREST;

       FOR J := 0 TO NR DO BEGIN

           TIU2[J] := ##( FOR L := 0 TO J SUM( TIA[L] * TIA[J-L] ) );
           TIU2INT[J+1] := H * TIU2[J] / ( J + 1 );
           FOR K := 0 TO N DO
               IF J < 3 THEN
                  TIU2Q[K,J+1] := H * ##( FOR L := 0 TO J
                               SUM( SPQ[K,L,I] * TIU2[J-L] ) ) / ( J + 1 )
               ELSE
                  TIU2Q[K,J+1] := H * ##( FOR L := 0 TO 3
                               SUM( SPQ[K,L,I] * TIU2[J-L] ) ) / ( J + 1 );

       END;

       (*  "OBERE" TERME BERECHNEN  *)

       FOR J := 1 TO NR DO BEGIN

           TIU2[NR+J] := ##( FOR L := J TO NR SUM( TIA[L] * TIA[NR+J-L] ) );
           TIU2INT[NR+J+1] := H * TIU2[J] / ( NR + J + 1 );
           FOR K := 0 TO N DO
               TIU2Q[K,NR+J+1] := H * ##( FOR L := 0 TO 3
                               SUM( SPQ[K,L,I] * TIU2[NR+J-L] ) ) / ( NR+J+1 );

       END;

       (*  "HCHSTE" TERME BERECHNEN  *)

       FOR J := 1 TO 3 DO
           FOR K := 0 TO N DO
               TIU2Q[K,2*NR+J+1] := H * ##( FOR L := J TO 3
                          SUM( SPQ[K,L,I] * TIU2[2*NR+J-L] ) ) / ( 2*NR+J+1 );

       (*  REIHENGLIEDER SUMMIEREN  *)

       VI_U2 := ##( VI_U2 + FOR J := 1 TO 2 * NR + 1 SUM( TIU2INT[J] ) );

       FOR K := 0 TO N DO
           VI_U2Q[K] := ##( VI_U2Q[K] + FOR J:=1 TO 2*NR+4 SUM( TIU2Q[K,J] ) );

       (*  WERTE AUSGEBEN  *)

       WRITELN( 'Restglied [' , NR : 2 , ']      = ' , RESTGLIED );
       WRITELN( 'Ableitungsrestglied = ' , ABLRESTGLIED );
       WRITELN;

       WRITELN( 'u (' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ' , REIHE );
       WRITELN( 'u''(' , I + 1 : 2 , '  / ' , M : 1 , ' ) = ', ABLREIHE );
       WRITELN;

   END; (* FOR , I *)

   VIU[1] := REIHE;
   VIU[2] := ABLREIHE;

END;

(*-------------------------------------------------------------------------*)

END.

(*-------------------------------------------------------------------------*)
(*                                                                         *)
(*  PASCAL-XSC - MODUL  IEP_IAWA                                           *)
(*                                                                         *)
(*  INTERVALL-ANFANGSWERTPROBLEMLSER FR DIE STURM-LIOUVILLE-GLEICHUNG    *)
(*                                                                         *)
(*-------------------------------------------------------------------------*)
